/* Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved. */

#ifndef PLUGIN_DEVICEINFO_DEVICEINFO_PROPERTIES_ABSTRACT_PLATFORM_SERVICE_H_
#define PLUGIN_DEVICEINFO_DEVICEINFO_PROPERTIES_ABSTRACT_PLATFORM_SERVICE_H_

#include "meta/ReflectableValue.h"
#include "sm/IBindable.h"

#include "DataStructures.h"
#include "Optional.h"
#include "SmartPointers.h"

#include <string>

namespace ignition
{
namespace deviceinfo
{
namespace services
{

typedef core::unordered_map<std::string, core::ReflectableValue> ServicePropertyMap;

class PlatformService : public javascript::sm::IBindable
{
DECLARE_BINDABLE_ID(PlatformService)

public:
	PlatformService(
			const std::string& id,
			const std::string& name,
			const core::optional<std::string>& version,
			bool enabled,
			const ServicePropertyMap& properties = ServicePropertyMap());

	std::string getId() const;
	std::string getName() const;
	std::string getVersion() const;
	bool isEnabled() const;
	core::ReflectableValue getProperty(const std::string& key) const;
	core::vector<std::string> getPropertyKeys() const;

private:
	const std::string _id;
	const std::string _name;
	const core::optional<std::string> _version;
	const bool _enabled;
	const ServicePropertyMap _properties;
};

typedef core::memory::SharedPtr<PlatformService> PlatformServicePtr;

} // namespace services
} // namespace deviceinfo
} // namespace ignition

#endif // PLUGIN_DEVICEINFO_DEVICEINFO_PROPERTIES_PLATFORM_SERVICE_H_
