/* Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved. */

#ifndef PLUGIN_NETWORK_HTTP_FACTORY_HTTP_REQUEST_FACTORY_H_
#define PLUGIN_NETWORK_HTTP_FACTORY_HTTP_REQUEST_FACTORY_H_

#include "http/HttpRequest.h"
#include "http/HttpResponse.h"
#include "http/HttpState.h"
#include "NetworkConfig.h"

#include "DataStructures.h"
#include "Function.h"

#include <string>

namespace ignition
{
namespace network
{
namespace http
{

typedef core::unordered_map<std::string, std::string> QueryStringParams;

typedef core::function<void(const std::string&)> SuccessCallbackType;
typedef core::function<
	void(const std::string&, HttpState::Type, HttpResponse::HttpStatus)> FailureCallbackType;

namespace factory
{

class PLUGINNETWORK_API HttpRequestFactory
{
public:
	HttpRequestFactory() {}
	virtual ~HttpRequestFactory() {}
	static HttpRequestFactory& get();
	virtual HttpRequestPtr create();
	virtual HttpRequestPtr create(const std::string& url, const QueryStringParams& params,
			SuccessCallbackType onSuccess, FailureCallbackType onFailure =
			[](const std::string& response, HttpState::Type state, HttpResponse::HttpStatus status){});
};

} // namespace factory
} // namespace http
} // namespace network
} // namespace ignition

#endif // PLUGIN_NETWORK_HTTP_FACTORY_HTTP_REQUEST_FACTORY_H_
