#version 100

// u_texture, specify the texture unit we want to sample from.
uniform lowp sampler2D u_texture;
uniform mediump vec4 transform;
// radius is the corner radius.
uniform mediump float radius;

uniform highp vec4 u_color_multiplier;

// pixel_pos is the position of this pixel in pixel unit which has been scaled to the desire size
// but hasn't been translated to the dst pos nor rotated. origin is top left.
varying mediump vec2 pixel_pos;
// v_texcoord is the texture coordinates.
varying mediump vec2 v_texcoord;

void main(void)
{
    mediump float t = 1.0;
    if (pixel_pos.y < radius)
    {
        if (pixel_pos.x < radius)
        {
            // Pixel is in the top left corner, calculate the distance
            // to the center of this corner (radius, radius)
            mediump float d = distance(pixel_pos, vec2(radius, radius));
            // Clip this pixel if it's outside the corner circle
            t = clamp(0.5 - d + radius, 0.0, 1.0);
        }
        else if (pixel_pos.x > transform.z - radius)
        {
            // Pixel is in the top right corner
            mediump float d = distance(pixel_pos, vec2(transform.z - radius, radius));
            t = clamp(0.5 - d + radius, 0.0, 1.0);
        }
    }
    else if (pixel_pos.y > transform.w - radius)
    {
        if (pixel_pos.x < radius)
        {
            // Pixel is in the bottom left corner
            mediump float d = distance(pixel_pos, vec2(radius, transform.w - radius));
            t = clamp(0.5 - d + radius, 0.0, 1.0);
        }
        else if (pixel_pos.x > transform.z - radius)
        {
            // Pixel is in the bottom right corner
            mediump float d = distance(pixel_pos, vec2(transform.z - radius, transform.w - radius));
            t = clamp(0.5 - d + radius, 0.0, 1.0);
        }
    }
    if (t > 0.0)
    {
        gl_FragColor = texture2D(u_texture, v_texcoord) * vec4(vec3(1.0),t);
    }
    else
    {
        // Texture lookup is very expensive,
        // we can skip it if the pixel is outside the corner circle
        gl_FragColor = vec4(0.0);
    }

    gl_FragColor = gl_FragColor * u_color_multiplier;
}